﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using System;
using System.Threading.Tasks;
using BeSafePlus.Models;
using BeSafePlus.Database;
using SkiaSharp;
using Microsoft.Maui.Dispatching;
using BeSafePlus.Sensors;
using System.Collections.Generic;
using System.Linq;

namespace BeSafePlus.ViewModels
{
    public partial class ActivityHistoryViewModel : ObservableObject
    {
        private readonly ILocalDatabase _database;

        private readonly SensorEventHandler _sensorEventHandler;

        private List<StepCount> _weeklyStepData;

        private List<StepCount> _monthlyStepData;

        private int _currentWeekOffset = 0;

        private int _currentMonthOffset = 0;

        private int _steps;

        private int _stepGoal = 5000; // Default Daily Goal

        private int _totalStepsThisWeek;

        private int _totalStepsThisMonth;
        public ActivityHistoryViewModel(ILocalDatabase database, SensorEventHandler sensorEventHandler)
        {
            _database = database;

            _weeklyStepData = new List<StepCount>();

            _monthlyStepData = new List<StepCount>();

            _sensorEventHandler = sensorEventHandler;

            _sensorEventHandler.UserStepEvent += UpdateSteps;

            PreviousWeekCommand = new RelayCommand(async () => await PreviousWeek());

            NextWeekCommand = new RelayCommand(async () => await NextWeek(), CanNavigateToNextWeek);
        }

        // Bindable properties
        public int AccumulatedSteps
        {
            get => _steps;
            set
            {
                if (SetProperty(ref _steps, value))
                {
                    OnPropertyChanged(nameof(Progress));
                }
            }
        }
        public float Progress => StepGoal > 0 ? (float) AccumulatedSteps / StepGoal : 0;
        public int StepGoal
        {
            get => _stepGoal;
            set
            {
                if (SetProperty(ref _stepGoal, value))
                {
                    OnPropertyChanged(nameof(Progress));
                }
            }
        }
        private async void UpdateSteps(int steps)
        {
            await MainThread.InvokeOnMainThreadAsync(async () =>
            {
                AccumulatedSteps = steps;

                var today = DateTime.Now.Date;

                var todayEntry = WeeklyStepData.FirstOrDefault(data => DateTime.Parse(data.Date).Date == today);

                if (todayEntry == null)
                {
                    WeeklyStepData.Add(new StepCount { Date = today.ToString("yyyy-MM-dd"), Steps = steps });
                }
                else if (todayEntry.Steps != steps)
                {
                    todayEntry.Steps = steps;
                }

                else
                {
                    OnPropertyChanged(nameof(WeeklyStepData));
                }

                TotalStepsThisWeek = WeeklyStepData.Sum(data => data.Steps);

                var todayMonthlyEntry = MonthlyStepData.FirstOrDefault(data => DateTime.Parse(data.Date).Date == today);

                if (todayMonthlyEntry == null)
                {
                    todayMonthlyEntry = new StepCount { Date = today.ToString("yyyy-MM-dd"), Steps = steps };

                    MonthlyStepData.Add(todayMonthlyEntry);
                }

                else
                {
                    todayMonthlyEntry.Steps = steps;
                }

                var firstDayOfMonth = new DateTime(today.Year, today.Month, 1);

                var lastDayOfMonth = firstDayOfMonth.AddMonths(1).AddDays(-1);

                TotalStepsThisMonth = MonthlyStepData
                    .Where(data => DateTime.Parse(data.Date) >= firstDayOfMonth && DateTime.Parse(data.Date) <= lastDayOfMonth)
                    .Sum(data => data.Steps);

                OnPropertyChanged(nameof(WeeklyStepData));

                OnPropertyChanged(nameof(TotalStepsThisWeek));

                OnPropertyChanged(nameof(AccumulatedSteps));

                OnPropertyChanged(nameof(TotalStepsThisMonth));

                await _database.CreateOrUpdateStepCount(steps);
            });
        }
        public async Task LoadStepGoal()
        {
            StepCount? stepCount = await _database.GetStepCountByDate(DateTime.Now);

            StepGoal = stepCount?.StepGoal ?? 5000;
        }
        public List<StepCount> WeeklyStepData
        {
            get => _weeklyStepData;
            private set => SetProperty(ref _weeklyStepData, value);
        }
        public int TotalStepsThisWeek
        {
            get => _totalStepsThisWeek;
            set => SetProperty(ref _totalStepsThisWeek, value);
        }
        public List<StepCount> MonthlyStepData
        {
            get => _monthlyStepData;
            private set => SetProperty(ref _monthlyStepData, value);
        }
        public int TotalStepsThisMonth
        {
            get => _totalStepsThisMonth;
            set
            {
                if (SetProperty(ref _totalStepsThisMonth, value))
                {
                    OnPropertyChanged(nameof(TotalStepsThisMonth));
                }
            }
        }
        public string WeekLabel => GetWeekLabel();
        public string MonthLabel => GetMonthLabel();
        public IRelayCommand PreviousWeekCommand { get; }
        public IRelayCommand NextWeekCommand { get; }
        public bool CanNavigateToNextWeek()
        {
            var today = DateTime.Now;

            var endOfCurrentWeek = today.AddDays(-(int)today.DayOfWeek + 7);

            var endOfSelectedWeek = today.AddDays(_currentWeekOffset * 7).AddDays(-(int)today.DayOfWeek + 7);

            return endOfSelectedWeek < endOfCurrentWeek;
        }
        public async Task LoadWeeklySteps()
        {
            var today = DateTime.Now.AddDays(_currentWeekOffset * 7);

            var startOfWeek = today.AddDays(-(int)today.DayOfWeek + 1);

            var endOfWeek = startOfWeek.AddDays(6);

            var newWeeklyData = new List<StepCount>();

            var monthCount = new Dictionary<int, int>();

            int totalSteps = 0;

            for (int i = 0; i < 7; i++)
            {
                var date = startOfWeek.AddDays(i);

                int monthKey = date.Month + date.Year * 100;

                if (!monthCount.ContainsKey(monthKey)) monthCount[monthKey] = 0;
                
                monthCount[monthKey]++;

                var stepData = await _database.GetStepCountByDate(date);

                if (stepData != null)
                {
                    newWeeklyData.Add(stepData);

                    totalSteps += stepData.Steps;
                }

                else
                {
                    newWeeklyData.Add(new StepCount { Steps = 0, Date = date.ToString("yyyy-MM-dd") });
                }
            }

            WeeklyStepData = newWeeklyData;

            TotalStepsThisWeek = totalSteps;

            int dominantMonthKey = monthCount.OrderByDescending(kv => kv.Value).First().Key;

            int dominantYear = dominantMonthKey / 100;

            int dominantMonth = dominantMonthKey % 100;

            var dominantMonthDate = new DateTime(dominantYear, dominantMonth, 1);

            var now = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);

            _currentMonthOffset = ((dominantMonthDate.Year - now.Year) * 12) + (dominantMonthDate.Month - now.Month);

            await LoadMonthlySteps();

            OnPropertyChanged(nameof(WeekLabel));

            OnPropertyChanged(nameof(MonthLabel));

            ((RelayCommand)NextWeekCommand).NotifyCanExecuteChanged();
        }
        public async Task LoadMonthlySteps()
        {
            var today = DateTime.Now.AddMonths(_currentMonthOffset);

            var firstDayOfMonth = new DateTime(today.Year, today.Month, 1);

            var lastDayOfMonth = firstDayOfMonth.AddMonths(1).AddDays(-1);

            int totalSteps = 0;

            var newMonthlyData = new List<StepCount>();

            for (var date = firstDayOfMonth; date <= lastDayOfMonth; date = date.AddDays(1))
            {
                var stepData = await _database.GetStepCountByDate(date);

                if (stepData != null)
                {
                    newMonthlyData.Add(stepData);

                    totalSteps += stepData.Steps;
                }

                else
                {
                    newMonthlyData.Add(new StepCount { Date = date.ToString("yyyy-MM-dd"), Steps = 0 });
                }
            }

            MonthlyStepData = newMonthlyData;

            TotalStepsThisMonth = totalSteps;
        }
        private string GetWeekLabel()
        {
            var today = DateTime.Now.AddDays(_currentWeekOffset * 7);

            var startOfWeek = today.AddDays(-(int)today.DayOfWeek + 1);

            var endOfWeek = startOfWeek.AddDays(6);

            return $"{startOfWeek:dd/MM} - {endOfWeek:dd/MM}";
        }
        private string GetMonthLabel()
        {
            var monthDate = DateTime.Now.AddMonths(_currentMonthOffset);

            return $"{monthDate:MMMM yyyy}";
        }
        public async Task PreviousWeek()
        {
            _currentWeekOffset--;

            await LoadWeeklySteps();
        }
        public async Task NextWeek()
        {
            if (_currentWeekOffset < 0)
            {
                _currentWeekOffset++;

                await LoadWeeklySteps();
            }
        }
        public async Task<int> GetStepGoal()
        {
            StepCount? stepCount = await _database.GetStepCountByDate(DateTime.Now);

            return stepCount?.StepGoal ?? 5000;
        }
        public async Task<int> GetStepCount()
        {
            StepCount? stepCount = await _database.GetStepCountByDate(DateTime.Now);

            return stepCount?.Steps ?? 0;
        }
    }
}