using BeSafePlus.ViewModels;
using Microsoft.Maui.Controls;
using Microcharts.Maui;
using SkiaSharp;
using System.ComponentModel;
using System.Collections.Generic;
using Microcharts;

namespace BeSafePlus
{
    public partial class ActivityHistoryPage : ContentPage
    {
        private ActivityHistoryViewModel _viewModel;

        private static readonly string[] DaysOfWeek = { "Mandag", "Tirsdag", "Onsdag", "Torsdag", "Fredag", "Lørdag", "Søndag" };
        public ActivityHistoryPage(ActivityHistoryViewModel vm)
        {
            InitializeComponent();

            BindingContext = _viewModel = vm;

            _viewModel.PropertyChanged += ViewModelOnPropertyChanged;
        }
        protected override async void OnAppearing()
        {
            base.OnAppearing();

            await Task.WhenAll(
                _viewModel.LoadStepGoal(),
                _viewModel.LoadWeeklySteps(),
                _viewModel.LoadMonthlySteps(),
                _viewModel.GetStepCount()
            );

            if (_viewModel.AccumulatedSteps == 0)
            {
                _viewModel.AccumulatedSteps = await _viewModel.GetStepCount();
            }

            UpdateCharts();
        }
        private async void ViewModelOnPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(_viewModel.WeeklyStepData) ||
                e.PropertyName == nameof(_viewModel.StepGoal) ||
                e.PropertyName == nameof(_viewModel.TotalStepsThisWeek) ||
                e.PropertyName == nameof(_viewModel.AccumulatedSteps))
            {
                UpdateCharts();
            }
        }
        private void UpdateCharts()
        {
            LoadBarChart();

            LoadLineChart();
        }
        private async void LoadBarChart()
        {
            if (_viewModel?.WeeklyStepData == null || !_viewModel.WeeklyStepData.Any()) return;

            var entries = new List<Microcharts.ChartEntry>();

            int stepGoal = await _viewModel.GetStepGoal();

            DateTime today = DateTime.Today;

            foreach (var stepData in _viewModel.WeeklyStepData)
            {
                if (DateTime.TryParse(stepData.Date, out DateTime stepDate))
                {
                    string dayName = GetDanishDayName(stepDate);

                    float percentage = (float) stepData.Steps / stepGoal;

                    var color = GetStepColor(stepData.Steps, stepGoal);

                    bool isToday = stepDate.Date == today;

                    entries.Add(new Microcharts.ChartEntry(stepData.Steps)
                    {
                        Label = isToday ? $"*{dayName}" : dayName,
                        ValueLabel = $"{stepData.Steps} ({percentage * 100:0}%)",
                        Color = color,
                        ValueLabelColor = SKColors.Black,
                        TextColor = SKColors.Black
                    });
                }
            }

            BarChart.Chart = new Microcharts.BarChart
            {
                Entries = entries,
                BackgroundColor = SKColors.White,
                LabelTextSize = 40,
                ValueLabelTextSize = 40,
                LabelColor = SKColors.Black,
                LabelOrientation = Microcharts.Orientation.Horizontal,
                MaxValue = stepGoal
            };
        }
        private void LoadLineChart()
        {
            if (_viewModel?.WeeklyStepData == null || !_viewModel.WeeklyStepData.Any()) return;

            var entries = new List<Microcharts.ChartEntry>();

            DateTime today = DateTime.Today;

            foreach (var stepData in _viewModel.WeeklyStepData)
            {
                if (DateTime.TryParse(stepData.Date, out DateTime stepDate))
                {
                    string dayName = GetDanishDayName(stepDate);

                    bool isToday = stepDate.Date == today;

                    entries.Add(new Microcharts.ChartEntry(stepData.Steps)
                    {
                        Label = isToday ? $"*{dayName}" : dayName,
                        ValueLabel = $"{stepData.Steps}",
                        Color = SKColors.Blue,
                        TextColor = SKColors.Black,
                        ValueLabelColor = SKColors.Black
                    });
                }
            }

            LineChart.Chart = new Microcharts.LineChart
            {
                Entries = entries,
                LineMode = Microcharts.LineMode.Straight,
                BackgroundColor = SKColors.White,
                LabelTextSize = 20,
                ValueLabelTextSize = 20,
                LabelColor = SKColors.Black,
                MaxValue = _viewModel.StepGoal
            };
        }
        private string GetDanishDayName(DateTime date)
        {
            int dayIndex = (int)date.DayOfWeek;

            return DaysOfWeek[dayIndex == 0 ? 6 : dayIndex - 1];
        }
        private SKColor GetStepColor(int steps, int stepGoal)
        {
            if (steps == 0) return SKColors.White;

            float percentage = (float) steps / stepGoal;

            return percentage switch
            {
                < 0.2f => SKColor.Parse("#FF0000"), // Rød
                < 0.4f => SKColor.Parse("#FF8000"), // Orange
                < 0.6f => SKColor.Parse("#FFFF00"), // Gul
                < 0.8f => SKColor.Parse("#00FF00"), // Lysegrøn
                < 1f => SKColor.Parse("#008000"), // Grøn
                _ => SKColor.Parse("#0000FF"), // Blå
            };
        }
    }
}